@@ -1,3 +1,4 @@
+import logging
 from typing import cast
 
 from linkedin_messaging import ChallengeException, LinkedInMessaging
@@ -28,21 +29,28 @@ VERIFY_URL = "https://www.linkedin.com/checkpoint/challenge/verify"
     help_text="See authentication status",
 )
 async def whoami(evt: CommandEvent):
-    if not evt.sender.client or not await evt.sender.client.logged_in():
+    assert evt.sender
+    user_profile = evt.sender.user_profile_cache
+    if user_profile is not None:
+        logging.debug("Cache hit on user_profile_cache")
+    elif not evt.sender.client or not await evt.sender.client.logged_in():
         await evt.reply("You are not logged in")
-    else:
-        user_profile = await evt.sender.client.get_user_profile()
-        if mini_profile := user_profile.mini_profile:
-            first = mini_profile.first_name
-            last = mini_profile.last_name
-            name = f"{first} {last}"
-        elif plain_id := user_profile.plain_id:
-            name = plain_id
-        else:
-            await evt.reply("You are not logged in")
-            return
+        return
+    assert evt.sender.client
 
-        await evt.reply(f"You are logged in as {name}")
+    user_profile = user_profile or await evt.sender.client.get_user_profile()
+    evt.sender.user_profile_cache = user_profile
+    if mini_profile := user_profile.mini_profile:
+        first = mini_profile.first_name
+        last = mini_profile.last_name
+        name = f"{first} {last}"
+    elif plain_id := user_profile.plain_id:
+        name = plain_id
+    else:
+        await evt.reply("You are not logged in")
+        return
+
+    await evt.reply(f"You are logged in as {name}")
 
 
 # region Login
