@@ -39,6 +39,7 @@ from mautrix.types import (
 )
 from mautrix.types.event.message import Format
 from mautrix.types.primitive import UserID
+from mautrix.util.message_send_checkpoint import MessageSendCheckpointStatus
 from mautrix.util.simple_lock import SimpleLock
 
 from . import puppet as p, user as u
@@ -148,9 +149,7 @@ class Portal(DBPortal, BasePortal):
         cls.matrix = bridge.matrix
         cls.invite_own_puppet_to_pm = cls.config["bridge.invite_own_puppet_to_pm"]
         NotificationDisabler.puppet_cls = p.Puppet
-        NotificationDisabler.config_enabled = cls.config[
-            "bridge.backfill.disable_notifications"
-        ]
+        NotificationDisabler.config_enabled = cls.config["bridge.backfill.disable_notifications"]
 
     # region DB conversion
 
@@ -191,9 +190,7 @@ class Portal(DBPortal, BasePortal):
     @property
     def main_intent(self) -> IntentAPI:
         if not self._main_intent:
-            raise ValueError(
-                "Portal must be postinit()ed before main_intent can be used"
-            )
+            raise ValueError("Portal must be postinit()ed before main_intent can be used")
         return self._main_intent
 
     @property
@@ -285,9 +282,7 @@ class Portal(DBPortal, BasePortal):
         for portal in portals:
             portal = cast(Portal, portal)
             try:
-                yield cls.by_li_thread_urn[
-                    (portal.li_thread_urn, portal.li_receiver_urn)
-                ]
+                yield cls.by_li_thread_urn[(portal.li_thread_urn, portal.li_receiver_urn)]
             except KeyError:
                 await portal.postinit()
                 yield portal
@@ -297,9 +292,7 @@ class Portal(DBPortal, BasePortal):
         portals = await super().all()
         for portal in cast(list[Portal], portals):
             try:
-                yield cls.by_li_thread_urn[
-                    (portal.li_thread_urn, portal.li_receiver_urn)
-                ]
+                yield cls.by_li_thread_urn[(portal.li_thread_urn, portal.li_receiver_urn)]
             except KeyError:
                 await portal.postinit()
                 yield portal
@@ -401,6 +394,10 @@ class Portal(DBPortal, BasePortal):
             return False
         self.topic = topic
 
+        if not self.topic and not self.topic_set:
+            self.topic_set = True
+            return False
+
         if self.mxid:
             try:
                 await self.main_intent.set_room_topic(self.mxid, self.topic or "")
@@ -454,19 +451,13 @@ class Portal(DBPortal, BasePortal):
                 participant_urn = conversation.entity_urn
             puppet = await p.Puppet.get_by_li_member_urn(participant_urn)
             await puppet.update_info(source, participant.messaging_member)
-            if (
-                self.is_direct
-                and self.li_other_user_urn == puppet.li_member_urn
-                and self.encrypted
-            ):
+            if self.is_direct and self.li_other_user_urn == puppet.li_member_urn:
                 changed = await self._update_name(puppet.name) or changed
                 changed = await self._update_photo_from_puppet(puppet) or changed
 
             if self.mxid:
                 if puppet.li_member_urn != self.li_receiver_urn or puppet.is_real_user:
-                    await puppet.intent_for(self).ensure_joined(
-                        self.mxid, bot=self.main_intent
-                    )
+                    await puppet.intent_for(self).ensure_joined(self.mxid, bot=self.main_intent)
 
         return changed
 
@@ -503,6 +494,14 @@ class Portal(DBPortal, BasePortal):
         except Exception:
             self.log.exception("Failed to update portal")
 
+    def _get_invite_content(self, double_puppet: Optional["p.Puppet"]) -> dict[str, Any]:
+        invite_content = {}
+        if double_puppet:
+            invite_content["fi.mau.will_auto_accept"] = True
+        if self.is_direct:
+            invite_content["is_direct"] = True
+        return invite_content
+
     async def _create_matrix_room(
         self,
         source: "u.User",
@@ -512,6 +511,10 @@ class Portal(DBPortal, BasePortal):
             await self._update_matrix_room(source, conversation)
             return self.mxid
 
+        # Update info before computing initial state because self.bridge_info depends on
+        # things that are set by this function.
+        await self.update_info(source, conversation)
+
         self.log.debug("Creating Matrix room")
         name: Optional[str] = None
         initial_state = [
@@ -528,7 +531,7 @@ class Portal(DBPortal, BasePortal):
                 "content": self.bridge_info,
             },
         ]
-        invites = [source.mxid]
+        invites = []
         if self.config["bridge.encryption.default"] and self.matrix.e2ee:
             self.encrypted = True
             initial_state.append(
@@ -540,8 +543,6 @@ class Portal(DBPortal, BasePortal):
             if self.is_direct:
                 invites.append(self.az.bot_mxid)
 
-        await self.update_info(source, conversation)
-
         if self.topic:
             initial_state.append(
                 {
@@ -568,11 +569,16 @@ class Portal(DBPortal, BasePortal):
         # created and the initial backfill finishing wouldn't be bridged before the
         # backfill messages.
         with self.backfill_lock:
+            creation_content = {}
+            if not self.config["bridge.federate_rooms"]:
+                creation_content["m.federate"] = False
+
             self.mxid = await self.main_intent.create_room(
                 name=name,
                 is_direct=self.is_direct,
                 initial_state=initial_state,
                 invitees=invites,
+                creation_content=creation_content,
             )
             if not self.mxid:
                 raise Exception("Failed to create room: no mxid returned")
@@ -581,17 +587,33 @@ class Portal(DBPortal, BasePortal):
                 try:
                     await self.az.intent.ensure_joined(self.mxid)
                 except Exception:
-                    self.log.warning(
-                        f"Failed to add bridge bot to new private chat {self.mxid}"
-                    )
+                    self.log.warning(f"Failed to add bridge bot to new private chat {self.mxid}")
 
             await self.save()
             self.log.debug(f"Matrix room created: {self.mxid}")
             self.by_mxid[self.mxid] = self
 
-            if not self.is_direct:
-                await self._update_participants(source, conversation)
-            else:
+            puppet = await p.Puppet.get_by_custom_mxid(source.mxid)
+            await self.main_intent.invite_user(
+                self.mxid, source.mxid, extra_content=self._get_invite_content(puppet)
+            )
+
+            if puppet:
+                try:
+                    if self.is_direct:
+                        await source.update_direct_chats({self.main_intent.mxid: [self.mxid]})
+                    await puppet.intent.join_room_by_id(self.mxid)
+                except MatrixError:
+                    self.log.debug(
+                        "Failed to join custom puppet into newly created portal",
+                        exc_info=True,
+                    )
+
+            if self.is_direct:
+                # Check if this is an ad. If it is, then set the room permissions such
+                # that the user is not allowed to send messages. (If they do send a
+                # message, the bridge would give an error since responding to such
+                # messages is not allowed.)
                 if (
                     (mm := conversation.participants[0].messaging_member)
                     and (mp := mm.mini_profile)
@@ -602,19 +624,7 @@ class Portal(DBPortal, BasePortal):
                         levels.events_default = 50
                         await self.main_intent.set_power_levels(self.mxid, levels)
 
-                puppet = await p.Puppet.get_by_custom_mxid(source.mxid)
-                if puppet:
-                    try:
-                        did_join = await puppet.intent.join_room_by_id(self.mxid)
-                        if did_join:
-                            await source.update_direct_chats(
-                                {self.main_intent.mxid: [self.mxid]}
-                            )
-                    except MatrixError:
-                        self.log.debug(
-                            "Failed to join custom puppet into newly created portal",
-                            exc_info=True,
-                        )
+            await self._update_participants(source, conversation)
 
             try:
                 await self.backfill(source, conversation, is_initial=True)
@@ -630,8 +640,13 @@ class Portal(DBPortal, BasePortal):
         source: "u.User",
         conversation: Optional[Conversation] = None,
     ):
-        await self.main_intent.invite_user(self.mxid, source.mxid, check_cache=False)
         puppet = await p.Puppet.get_by_custom_mxid(source.mxid)
+        await self.main_intent.invite_user(
+            self.mxid,
+            source.mxid,
+            check_cache=False,
+            extra_content=self._get_invite_content(puppet),
+        )
         if puppet and puppet.is_real_user:
             await puppet.intent.ensure_joined(self.mxid)
         await self.update_info(source, conversation)
@@ -682,17 +697,11 @@ class Portal(DBPortal, BasePortal):
         if not is_initial and conversation and len(conversation.events) > 0:
             last_active = conversation.events[-1].created_at
 
-        most_recent = await DBMessage.get_most_recent(
-            self.li_thread_urn, self.li_receiver_urn
-        )
+        most_recent = await DBMessage.get_most_recent(self.li_thread_urn, self.li_receiver_urn)
         if most_recent and is_initial:
-            self.log.debug(
-                f"Not backfilling {self.li_urn_log}: already bridged messages found"
-            )
+            self.log.debug(f"Not backfilling {self.li_urn_log}: already bridged messages found")
         elif (not most_recent or not most_recent.timestamp) and not is_initial:
-            self.log.debug(
-                f"Not backfilling {self.li_urn_log}: no most recent message found"
-            )
+            self.log.debug(f"Not backfilling {self.li_urn_log}: no most recent message found")
         elif last_active and most_recent and most_recent.timestamp >= last_active:
             self.log.debug(
                 f"Not backfilling {self.li_urn_log}: last activity is equal to most "
@@ -726,9 +735,7 @@ class Portal(DBPortal, BasePortal):
         else:
             before_timestamp = datetime.now()
 
-        self.log.debug(
-            f"Fetching up to {limit} messages through {source.li_member_urn}"
-        )
+        self.log.debug(f"Fetching up to {limit} messages through {source.li_member_urn}")
 
         while limit is None or len(messages) < limit:
             result = await source.client.get_conversation(
@@ -778,9 +785,7 @@ class Portal(DBPortal, BasePortal):
                     or not (mp := mm.mini_profile)
                     or not (entity_urn := mp.entity_urn)
                 ):
-                    self.log.error(
-                        "No entity_urn found on message mini_profile!", message
-                    )
+                    self.log.error("No entity_urn found on message mini_profile!", message)
                     continue
                 member_urn = entity_urn
                 if member_urn == URN("UNKNOWN"):
@@ -815,13 +820,10 @@ class Portal(DBPortal, BasePortal):
 
     async def handle_matrix_leave(self, user: "u.User"):
         if self.is_direct:
-            self.log.info(
-                f"{user.mxid} left private chat portal with {self.li_other_user_urn}"
-            )
+            self.log.info(f"{user.mxid} left private chat portal with {self.li_other_user_urn}")
             if user.li_member_urn == self.li_receiver_urn:
                 self.log.info(
-                    f"{user.mxid} was the recipient of this portal. "
-                    "Cleaning up and deleting..."
+                    f"{user.mxid} was the recipient of this portal. " "Cleaning up and deleting..."
                 )
                 await self.cleanup_and_delete()
         else:
@@ -833,14 +835,34 @@ class Portal(DBPortal, BasePortal):
         message: MessageEventContent,
         event_id: EventID,
     ):
+        assert self.mxid
+
+        exception: Optional[Exception] = None
+        status = MessageSendCheckpointStatus.PERM_FAILURE
         try:
             await self._handle_matrix_message(sender, message, event_id)
+        except NotImplementedError as e:
+            await self._send_bridge_error(str(e), certain_failure=True)
+            exception = e
+            status = MessageSendCheckpointStatus.UNSUPPORTED
         except Error as e:
             self.log.exception(f"Failed handling {event_id}: {e.to_json()}")
             await self._send_bridge_error(e.to_json())
+            exception = e
         except Exception as e:
             self.log.exception(f"Failed handling {event_id}")
             await self._send_bridge_error(str(e))
+            exception = e
+
+        if exception:
+            sender.send_remote_checkpoint(
+                status,
+                event_id,
+                self.mxid,
+                EventType.ROOM_MESSAGE,
+                message.msgtype,
+                error=exception,
+            )
 
     async def _handle_matrix_message(
         self,
@@ -848,15 +870,7 @@ class Portal(DBPortal, BasePortal):
         message: MessageEventContent,
         event_id: EventID,
     ):
-        if message.get(
-            self.az.real_user_content_key, False
-        ) and await p.Puppet.get_by_custom_mxid(sender.mxid):
-            self.log.debug(
-                f"Ignoring puppet-sent message by confirmed puppet user {sender.mxid}"
-            )
-            return
-
-        if message.msgtype in (MessageType.TEXT, MessageType.NOTICE, MessageType.EMOTE):
+        if message.msgtype.is_text:
             await self._handle_matrix_text(
                 event_id,
                 sender,
@@ -869,18 +883,14 @@ class Portal(DBPortal, BasePortal):
                 cast(MediaMessageEventContent, message),
             )
         else:
-            self.log.warning(f"Unsupported msgtype {message.msgtype} in {event_id}")
-            await self._send_bridge_error(
-                f"messages of {message.msgtype} are not supported.",
-                certain_failure=True,
-            )
-            return
+            raise NotImplementedError(f"Messages of type {message.msgtype} are not supported.")
 
     async def _send_linkedin_message(
         self,
         event_id: EventID,
         sender: "u.User",
         message_create: MessageCreate,
+        message_type: MessageType,
     ) -> DBMessage:
         assert self.mxid
         assert self.li_receiver_urn
@@ -892,6 +902,13 @@ class Portal(DBPortal, BasePortal):
             if not resp.value or not resp.value.event_urn:
                 raise Exception("Response value was None.")
 
+            sender.send_remote_checkpoint(
+                MessageSendCheckpointStatus.SUCCESS,
+                event_id,
+                self.mxid,
+                EventType.ROOM_MESSAGE,
+                message_type,
+            )
             message = DBMessage(
                 mxid=event_id,
                 mx_room=self.mxid,
@@ -914,10 +931,13 @@ class Portal(DBPortal, BasePortal):
         message: TextMessageEventContent,
     ):
         assert sender.client
-        message_create = await matrix_to_linkedin(
-            message, sender, self.main_intent, self.log
+        message_create = await matrix_to_linkedin(message, sender, self.main_intent, self.log)
+        await self._send_linkedin_message(
+            event_id,
+            sender,
+            message_create,
+            message.msgtype,
         )
-        await self._send_linkedin_message(event_id, sender, message_create)
 
     async def _handle_matrix_media(
         self,
@@ -946,14 +966,13 @@ class Portal(DBPortal, BasePortal):
         else:
             return
 
-        attachment = await sender.client.upload_media(
-            data, message.body, message.info.mimetype
-        )
+        attachment = await sender.client.upload_media(data, message.body, message.info.mimetype)
         attachment.media_type = attachment.media_type or ""
         await self._send_linkedin_message(
             event_id,
             sender,
             MessageCreate(AttributedBody(), attachments=[attachment]),
+            message.msgtype,
         )
 
     async def handle_matrix_redaction(
@@ -962,30 +981,49 @@ class Portal(DBPortal, BasePortal):
         event_id: EventID,
         redaction_event_id: EventID,
     ):
+        try:
+            await self._handle_matrix_redaction(sender, event_id)
+        except Exception as e:
+            sender.send_remote_checkpoint(
+                MessageSendCheckpointStatus.PERM_FAILURE,
+                redaction_event_id,
+                self.mxid,
+                EventType.ROOM_REDACTION,
+                error=e,
+            )
+        else:
+            sender.send_remote_checkpoint(
+                MessageSendCheckpointStatus.SUCCESS,
+                redaction_event_id,
+                self.mxid,
+                EventType.ROOM_REDACTION,
+            )
+            await self._send_delivery_receipt(redaction_event_id)
+
+    async def _handle_matrix_redaction(self, sender: "u.User", event_id: EventID):
         if not self.mxid or not sender.client:
             return
 
         message = await DBMessage.get_by_mxid(event_id, self.mxid)
         if message:
-            try:
-                await message.delete()
-                await sender.client.delete_message(
-                    self.li_thread_urn, message.li_message_urn
-                )
-            except Exception:
-                self.log.exception("Delete message failed")
+            self.log.info(f"Deleting {message.li_message_urn} in {self.li_thread_urn}")
+            await message.delete()
+            await sender.client.delete_message(self.li_thread_urn, message.li_message_urn)
+            return
 
         reaction = await DBReaction.get_by_mxid(event_id, self.mxid)
         if reaction:
-            try:
-                await reaction.delete()
-                await sender.client.remove_emoji_reaction(
-                    self.li_thread_urn, reaction.li_message_urn, emoji=reaction.reaction
-                )
-            except Exception:
-                self.log.exception("Removing reaction failed")
+            self.log.info(
+                f"Deleting reaction {reaction.reaction} from {reaction.li_message_urn}"
+                f" in {self.li_thread_urn}"
+            )
+            await reaction.delete()
+            await sender.client.remove_emoji_reaction(
+                self.li_thread_urn, reaction.li_message_urn, emoji=reaction.reaction
+            )
+            return
 
-        await self._send_delivery_receipt(redaction_event_id)
+        raise Exception("No message or reaction found for redaction")
 
     async def handle_matrix_reaction(
         self,
@@ -1002,19 +1040,29 @@ class Portal(DBPortal, BasePortal):
                 self.log.debug(f"Ignoring reaction to unknown event {reacting_to}")
                 return
 
-            await sender.client.add_emoji_reaction(
-                self.li_thread_urn, message.li_message_urn, reaction
-            )
-            await DBReaction(
-                mxid=event_id,
-                mx_room=message.mx_room,
-                li_message_urn=message.li_message_urn,
-                li_receiver_urn=self.li_receiver_urn,
-                li_sender_urn=sender.li_member_urn,
-                reaction=reaction,
-            ).insert()
-
-        await self._send_delivery_receipt(event_id)
+            try:
+                await sender.client.add_emoji_reaction(
+                    self.li_thread_urn, message.li_message_urn, reaction
+                )
+            except Exception:
+                self.log.exception("Failed to send emoji reaction")
+                raise
+            else:
+                sender.send_remote_checkpoint(
+                    MessageSendCheckpointStatus.SUCCESS,
+                    event_id,
+                    self.mxid,
+                    EventType.REACTION,
+                )
+                await DBReaction(
+                    mxid=event_id,
+                    mx_room=message.mx_room,
+                    li_message_urn=message.li_message_urn,
+                    li_receiver_urn=self.li_receiver_urn,
+                    li_sender_urn=sender.li_member_urn,
+                    reaction=reaction,
+                ).insert()
+                await self._send_delivery_receipt(event_id)
 
     # endregion
 
@@ -1040,8 +1088,7 @@ class Portal(DBPortal, BasePortal):
                 != Membership.JOIN
             ):
                 self.log.warning(
-                    f"Ignoring own {mid} in private chat because own puppet is not in"
-                    " room."
+                    f"Ignoring own {mid} in private chat because own puppet is not in" " room."
                 )
                 return False
         return True
@@ -1061,18 +1108,12 @@ class Portal(DBPortal, BasePortal):
                     and (nu := cc.conversation_name_update_content)
                 ):
                     await self._update_name(nu.new_name)
-            elif (
-                (ec := message.event_content)
-                and (me := ec.message_event)
-                and me.recalled_at
-            ):
+            elif (ec := message.event_content) and (me := ec.message_event) and me.recalled_at:
                 await self._handle_linkedin_message_deletion(sender, message)
             else:
                 await self._handle_linkedin_message(source, sender, message)
         except Exception as e:
-            self.log.exception(
-                f"Error handling LinkedIn message {message.entity_urn}: {e}"
-            )
+            self.log.exception(f"Error handling LinkedIn message {message.entity_urn}: {e}")
 
     async def _handle_linkedin_message(
         self,
@@ -1090,21 +1131,16 @@ class Portal(DBPortal, BasePortal):
         event_ids: list[EventID] = []
         async with self.require_send_lock(sender.li_member_urn):
             if li_message_urn in self._dedup:
-                self.log.trace(
-                    f"Not handling message {li_message_urn}, found ID in dedup queue"
-                )
+                self.log.trace(f"Not handling message {li_message_urn}, found ID in dedup queue")
                 # Return here, because it is in the process of being handled.
                 return
             self._dedup.appendleft(li_message_urn)
 
             # Check database for duplicates
-            dbm = await DBMessage.get_all_by_li_message_urn(
-                li_message_urn, self.li_receiver_urn
-            )
+            dbm = await DBMessage.get_all_by_li_message_urn(li_message_urn, self.li_receiver_urn)
             if len(dbm) > 0:
                 self.log.debug(
-                    f"Not handling message {li_message_urn}, found duplicate in "
-                    "database."
+                    f"Not handling message {li_message_urn}, found duplicate in " "database."
                 )
                 # Don't return here because we may need to update the reactions.
                 message_exists = True
@@ -1118,9 +1154,7 @@ class Portal(DBPortal, BasePortal):
                 if not mxid:
                     # Failed to create
                     return
-            if not await self._bridge_own_message_pm(
-                source, sender, f"message {li_message_urn}"
-            ):
+            if not await self._bridge_own_message_pm(source, sender, f"message {li_message_urn}"):
                 return
 
             if (
@@ -1129,9 +1163,7 @@ class Portal(DBPortal, BasePortal):
                 and intent != sender.intent
                 and intent not in self._backfill_leave
             ):
-                self.log.debug(
-                    "Adding %s's default puppet to room for backfilling", sender.mxid
-                )
+                self.log.debug("Adding %s's default puppet to room for backfilling", sender.mxid)
                 await self.main_intent.invite_user(self.mxid, intent.mxid)
                 await intent.ensure_joined(self.mxid)
                 self._backfill_leave.add(intent)
@@ -1221,9 +1253,7 @@ class Portal(DBPortal, BasePortal):
                     return
 
                 # Save all of the messages in the database.
-                self.log.debug(
-                    f"Handled LinkedIn message {li_message_urn} -> {event_ids}"
-                )
+                self.log.debug(f"Handled LinkedIn message {li_message_urn} -> {event_ids}")
                 await DBMessage.bulk_create(
                     li_message_urn=li_message_urn,
                     li_thread_urn=self.li_thread_urn,
@@ -1291,9 +1321,7 @@ class Portal(DBPortal, BasePortal):
             sender = await p.Puppet.get_by_li_member_urn(reactor.reactor_urn)
             intent = sender.intent_for(self)
 
-            mxid = await intent.react(
-                self.mxid, reaction_event_id, reaction_summary.emoji
-            )
+            mxid = await intent.react(self.mxid, reaction_event_id, reaction_summary.emoji)
             mxids.append(mxid)
 
             self.log.debug(
@@ -1438,9 +1466,7 @@ class Portal(DBPortal, BasePortal):
             file_data, decryption_info = encrypt_attachment(file_data)
             upload_mime_type = "application/octet-stream"
             filename = None
-        url = await intent.upload_media(
-            file_data, mime_type=upload_mime_type, filename=filename
-        )
+        url = await intent.upload_media(file_data, mime_type=upload_mime_type, filename=filename)
         if decryption_info:
             decryption_info.url = url
         return url, info, decryption_info
@@ -1451,17 +1477,11 @@ class Portal(DBPortal, BasePortal):
         sender: "p.Puppet",
         event: RealTimeEventStreamEvent,
     ):
-        if (
-            not event.event_urn
-            or not self.li_receiver_urn
-            or not event.reaction_summary
-        ):
+        if not event.event_urn or not self.li_receiver_urn or not event.reaction_summary:
             return
         reaction = event.reaction_summary.emoji
         # Make up a URN for the reacton for dedup purposes
-        dedup_id = URN(
-            f"({event.event_urn.id_str()},{sender.li_member_urn.id_str()},{reaction})"
-        )
+        dedup_id = URN(f"({event.event_urn.id_str()},{sender.li_member_urn.id_str()},{reaction})")
         async with self.optional_send_lock(sender.li_member_urn):
             if dedup_id in self._dedup:
                 return
@@ -1481,16 +1501,12 @@ class Portal(DBPortal, BasePortal):
                 )
                 return
 
-        if not await self._bridge_own_message_pm(
-            source, sender, f"reaction to {event.event_urn}"
-        ):
+        if not await self._bridge_own_message_pm(source, sender, f"reaction to {event.event_urn}"):
             return
 
         intent = sender.intent_for(self)
 
-        message = await DBMessage.get_by_li_message_urn(
-            event.event_urn, self.li_receiver_urn
-        )
+        message = await DBMessage.get_by_li_message_urn(event.event_urn, self.li_receiver_urn)
         if not message:
             self.log.debug(f"Ignoring reaction to unknown message {event.event_urn}")
             return
