@@ -12,6 +12,7 @@ from typing import (
 )
 
 import aiohttp
+from aiohttp.client_exceptions import TooManyRedirects
 from linkedin_messaging import LinkedInMessaging, URN
 from linkedin_messaging.api_objects import (
     Conversation,
@@ -20,7 +21,7 @@ from linkedin_messaging.api_objects import (
     RealTimeEventStreamEvent,
     UserProfileResponse,
 )
-from mautrix.bridge import async_getter_lock, BaseUser, BridgeState
+from mautrix.bridge import async_getter_lock, BaseUser
 from mautrix.errors import MNotFound
 from mautrix.types import (
     PushActionType,
@@ -29,7 +30,7 @@ from mautrix.types import (
     RoomID,
     UserID,
 )
-from mautrix.util.bridge_state import BridgeStateEvent
+from mautrix.util.bridge_state import BridgeState, BridgeStateEvent
 from mautrix.util.opt_prometheus import async_time, Gauge, Summary
 from mautrix.util.simple_lock import SimpleLock
 
@@ -58,6 +59,7 @@ class User(DBUser, BaseUser):
 
     _is_connected: Optional[bool]
     _is_logged_in: Optional[bool]
+    _is_logging_out: Optional[bool]
     _is_refreshing: bool
     _notice_room_lock: asyncio.Lock
     _notice_send_lock: asyncio.Lock
@@ -84,6 +86,7 @@ class User(DBUser, BaseUser):
             self.permission_level,
         ) = self.config.get_permissions(mxid)
         self._is_logged_in = None
+        self._is_logging_out = None
         self._is_connected = None
         self._connection_time = time.monotonic()
         self._prev_thread_sync = -10
@@ -104,9 +107,7 @@ class User(DBUser, BaseUser):
         cls.config = bridge.config
         cls.az = bridge.az
         cls.loop = bridge.loop
-        cls.temp_disconnect_notices = bridge.config[
-            "bridge.temporary_disconnect_notices"
-        ]
+        cls.temp_disconnect_notices = bridge.config["bridge.temporary_disconnect_notices"]
         return (user.load_session() async for user in cls.all_logged_in())
 
     @property
@@ -191,14 +192,26 @@ class User(DBUser, BaseUser):
     ) -> bool:
         if self._is_logged_in and not _override:
             return True
-        if not self.client or not await self.client.logged_in():
+        if not self.client:
             return False
 
-        self.user_profile_cache = await self.client.get_user_profile()
-        if (mp := self.user_profile_cache.mini_profile) and mp.entity_urn:
-            self.li_member_urn = mp.entity_urn
-        else:
+        try:
+            self.user_profile_cache = await self.client.get_user_profile()
+        except TooManyRedirects as e:
+            await self.push_bridge_state(
+                BridgeStateEvent.BAD_CREDENTIALS,
+                message=str(e),
+            )
             return False
+        except Exception as e:
+            await self.push_bridge_state(BridgeStateEvent.UNKNOWN_ERROR, message=str(e))
+            return False
+        else:
+            if (mp := self.user_profile_cache.mini_profile) and mp.entity_urn:
+                self.li_member_urn = mp.entity_urn
+            else:
+                return False
+
         await self.push_bridge_state(BridgeStateEvent.CONNECTING)
 
         self.log.info("Loaded session successfully")
@@ -260,6 +273,7 @@ class User(DBUser, BaseUser):
     async def logout(self):
         self.log.info("Logging out")
         self._is_logged_in = False
+        self._is_logging_out = True
         self.stop_listen()
         if self.client:
             self.log.info("Logging out the client.")
@@ -279,9 +293,8 @@ class User(DBUser, BaseUser):
         self.user_profile_cache = None
         self.li_member_urn = None
         self.notice_room = None
-        self._is_logged_in = False
-        self.stop_listen()
         await self.save()
+        self._is_logging_out = False
 
     # endregion
 
@@ -298,9 +311,7 @@ class User(DBUser, BaseUser):
     @async_time(METRIC_SYNC_THREADS)
     async def sync_threads(self):
         if self._prev_thread_sync + 10 > time.monotonic():
-            self.log.debug(
-                "Previous thread sync was less than 10 seconds ago, not re-syncing"
-            )
+            self.log.debug("Previous thread sync was less than 10 seconds ago, not re-syncing")
             return
         self._prev_thread_sync = time.monotonic()
         try:
@@ -332,9 +343,7 @@ class User(DBUser, BaseUser):
                     await self._sync_thread(conversation)
                 except Exception:
                     self.user_profile_cache = None
-                    self.log.exception(
-                        f"Failed to sync thread {conversation.entity_urn}"
-                    )
+                    self.log.exception(f"Failed to sync thread {conversation.entity_urn}")
                 synced_threads += 1
 
             await self.update_direct_chats()
@@ -357,11 +366,7 @@ class User(DBUser, BaseUser):
         li_other_user_urn = None
         if not conversation.group_chat:
             other_user = conversation.participants[0]
-            if (
-                (mm := other_user.messaging_member)
-                and (mp := mm.mini_profile)
-                and mp.entity_urn
-            ):
+            if (mm := other_user.messaging_member) and (mp := mm.mini_profile) and mp.entity_urn:
                 li_other_user_urn = mp.entity_urn
                 if li_other_user_urn == URN("UNKNOWN"):
                     li_other_user_urn = conversation.entity_urn
@@ -424,9 +429,7 @@ class User(DBUser, BaseUser):
                     self.log.debug("Cache hit on user_profile_cache")
                 user_profile = user_profile or await user.client.get_user_profile()
                 if mp := user_profile.mini_profile:
-                    state.remote_name = " ".join(
-                        n for n in [mp.first_name, mp.last_name] if n
-                    )
+                    state.remote_name = " ".join(n for n in [mp.first_name, mp.last_name] if n)
             except Exception:
                 self.user_profile_cache = None
                 pass
@@ -443,6 +446,20 @@ class User(DBUser, BaseUser):
             self.log.info("Listener task cancelled")
         if self.client and self._is_logged_in and not self.shutdown:
             self.start_listen()
+        else:
+            # This most likely means that the bridge is being stopped/restarted. But,
+            # occasionally, the user gets logged out. In these cases, we want to reset
+            # _is_logged_in so the next whoami call does a full call out to LinkedIn to
+            # detect whether the user is logged in.
+            self.log.warn("No client, not logged in, or shutdown. Not reconnecting.")
+            if (
+                not self._is_logged_in
+                and not self._is_logging_out
+                and self.client
+                and not self.shutdown
+            ):
+                self.log.warn("Logged out, but not by a logout call, sending bad credentials.")
+                asyncio.create_task(self.push_bridge_state(BridgeStateEvent.BAD_CREDENTIALS))
 
     listener_event_handlers_created: bool = False
     listener_task_i: int = 0
@@ -458,16 +475,10 @@ class User(DBUser, BaseUser):
     async def _try_listen(self):
         assert self.client
         if not self.listener_event_handlers_created:
-            self.client.add_event_listener(
-                "ALL_EVENTS", self.handle_linkedin_stream_event
-            )
-            self.client.add_event_listener(
-                "STREAM_ERROR", self.handle_linkedin_listener_error
-            )
+            self.client.add_event_listener("ALL_EVENTS", self.handle_linkedin_stream_event)
+            self.client.add_event_listener("STREAM_ERROR", self.handle_linkedin_listener_error)
             self.client.add_event_listener("event", self.handle_linkedin_event)
-            self.client.add_event_listener(
-                "reactionAdded", self.handle_linkedin_reaction_added
-            )
+            self.client.add_event_listener("reactionAdded", self.handle_linkedin_reaction_added)
             self.listener_event_handlers_created = True
         await self.client.start_listener()
 
